package cn.ecpark.service.usergw.biz.filters.factory;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

import com.alibaba.fastjson.JSON;

import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.cloud.gateway.support.DefaultServerRequest;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.core.io.buffer.DataBufferUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.codec.HttpMessageReader;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.util.MultiValueMap;
import org.springframework.web.reactive.function.BodyExtractor;
import org.springframework.web.reactive.function.BodyInserter;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.server.ServerRequest;

import cn.ecpark.service.usergw.biz.filters.bean.GenericServicePool;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;;

@Component
@Slf4j
public class DubboGatewayFilterFactory extends AbstractGatewayFilterFactory<DubboGatewayFilterFactory.Config> {

    @Autowired
    private ApplicationContext appContext;

    /**
     * DUBBO_URI key.
     */
    public static final String DUBBO_URI = "dubbo_uri";
    public static HashMap<String, List<String>> mehtods = new HashMap<>();

    public DubboGatewayFilterFactory() {
        super(Config.class);
    }

    @Override
    public List<String> shortcutFieldOrder() {
        return Arrays.asList(DUBBO_URI);
    }

    @Override
    public GatewayFilter apply(Config config) {
        String uri = config.dubboUri;

        return (exchange, chain) -> {

            GenericServicePool gsPool = appContext.getBean(GenericServicePool.class);
            GenericService gs = gsPool.get(uri);

            // ServerRequest serverRequest = new DefaultServerRequest(exchange);
            // Mono<String> modifiedBody = serverRequest.bodyToMono(String.class).map(s -> {
            //     log.info("original text:{}", s);
            //     /**
            //      * do your decrypt() here and convert to String.class or you can change the
            //      * output class to change format
            //      */
            //     return "12 " + s;
            // });

            // modifiedBody.subscribe(databuferr -> {
            //     log.info(databuferr);
            // });

            ServerHttpRequest req = exchange.getRequest();
            HttpHeaders headers = req.getHeaders();

            List<String> methodString = headers.get("method");
            List<String> params = headers.get("params");

            if(methodString.size() != 0) {
                // special
                List<String> paramTypes = mehtods.get(methodString.get(0));
                if(paramTypes != null) {

                    int paramsSize = 0;
                    if(params != null) {
                        paramsSize = params.size();
                    }

                    if(paramTypes.size() == paramsSize) {
                        Object result = null;
                        if(paramsSize == 0) {
                            result = gs.$invoke(methodString.get(0), new String[]{}, new Object[]{}); 
                        } else {
                            result = gs.$invoke(methodString.get(0),  Arrays.copyOf(paramTypes.toArray(), paramTypes.size(), String[].class), params.toArray());
                        }
                        
                        ServerHttpResponse response = exchange.getResponse();
                        if (result != null) {
                            return response.writeWith(Mono
                                    .just(response.bufferFactory().wrap(ByteBuffer.wrap(JSON.toJSONString(result).getBytes()))));
                        } 
                        return response.setComplete();

                    }else {
                        log.warn("paramTypes.size {} is not equals to params size {}", paramTypes.size(), params.size());
                    }
                } else {
                    log.warn("mehtod: {} is not exist or not allowed", methodString.get(0));
                }

            } else {
                log.info("queryParams.get(\"method\") is null");
            }


            return chain.filter(exchange);
        };
    }

    public static class Config {

        private String dubboUri;

        public String getDubboUri() {
            return dubboUri;
        }

        public void setDubboUri(String dubboUri) {
            this.dubboUri = dubboUri;
        }

    }

}